# -*- coding: utf-8 -*-
# This file is part of cernopendata-client.
#
# Copyright (C) 2020 CERN.
#
# cernopendata-client is free software; you can redistribute it and/or modify
# it under the terms of the GPLv3 license; see LICENSE file for more details.

"""cernopendata-client utility functions."""

import click
import sys

from .printer import display_message


def parse_parameters(filter_input):
    """Return parsed filter parameters.

    :param filter_input: Tuple of filters input
    :type filter_input: tuple

    :return: List of filters separated by commas
    :rtype: list
    """
    try:
        filters = " ".join(filter_input).split(",")
        return filters
    except Exception:
        display_message(
            msg_type="error",
            msg="{} - Wrong input format".format(filter_input),
        )
        sys.exit(2)



# -*- coding: utf-8 -*-
#
# This file is part of cernopendata-client.
#
# Copyright (C) 2020 CERN.
#
# cernopendata-client is free software; you can redistribute it and/or modify
# it under the terms of the GPLv3 license; see LICENSE file for more details.

"""cernopendata-client validator test."""

import click
import pytest

from cernopendata_client.validator import (
    validate_recid,
    validate_server,
    validate_range,
    validate_directory,
    validate_retry_limit,
    validate_retry_sleep,
)


def test_validate_recid():
    """Test validate_recid()."""
    pytest.raises(SystemExit, validate_recid, -1)
    pytest.raises(SystemExit, validate_recid, 0)
    pytest.raises(SystemExit, validate_recid, None)
    assert validate_recid(1) is True


def test_validate_server():
    """Test validate_server()."""
    assert validate_server("http://opendata.cern.ch") is True
    assert validate_server("http://opendata-dev.cern.ch") is True
    assert validate_server("http://0.0.0.0:5000") is True
    assert validate_server("https://opendata.cern.ch") is True
    pytest.raises(SystemExit, validate_server, "root://opendata.cern.ch")
    pytest.raises(SystemExit, validate_server, "opendata.cern.ch")


def test_validate_range():
    """Test validate_range()."""
    assert validate_range(range="3-9", count=10) is True
    assert validate_range(range="1-5", count=8) is True
    pytest.raises(SystemExit, validate_range, "0-9", 5)
    pytest.raises(SystemExit, validate_range, "0", 5)
    pytest.raises(SystemExit, validate_range, "1-9", 5)
    pytest.raises(SystemExit, validate_range, "3-2", 5)
    pytest.raises(SystemExit, validate_range, "3,2", 5)


def test_validate_directory():
    """Test validate_directory()."""
    assert validate_directory(directory="/eos/opendata/cms/validated-runs/") is True
    pytest.raises(
        SystemExit,
        validate_directory,
        "root://eospublic.cern.ch//eos/opendata/cms/validated-runs",
    )
    pytest.raises(SystemExit, validate_directory, 90)


def test_validate_retry_limit():
    """Test validate_retry_limit()."""
    pytest.raises(SystemExit, validate_retry_limit, -1)
    pytest.raises(SystemExit, validate_retry_limit, 0)
    pytest.raises(SystemExit, validate_retry_limit, None)
    assert validate_recid(1) is True


def test_validate_retry_sleep():
    """Test validate_retry_sleep()."""
    pytest.raises(SystemExit, validate_retry_sleep, -1)
    pytest.raises(SystemExit, validate_retry_sleep, 0)
    pytest.raises(SystemExit, validate_retry_sleep, None)
    assert validate_recid(1) is True



# -*- coding: utf-8 -*-
# This file is part of cernopendata-client.
#
# Copyright (C) 2020, 2021 CERN.
#
# cernopendata-client is free software; you can redistribute it and/or modify
# it under the terms of the GPLv3 license; see LICENSE file for more details.

"""cernopendata-client file downloading related utilities."""

from __future__ import print_function
import sys
import os
import re
import time

try:
    import requests

    requests_available = True
except ImportError:
    requests_available = False

try:
    import pycurl

    pycurl_available = True
except ImportError:
    pycurl_available = False

try:
    from XRootD import client as xrootdclient

    xrootd_available = True
except ImportError:
    xrootd_available = False


from .validator import validate_range
from .printer import display_message
from .verifier import get_file_checksum
from .config import (
    DOWNLOAD_ERROR_PAGE,
    DOWNLOAD_ENGINE_PROTOCOL_HTTP_MAP,
    DOWNLOAD_ENGINE_PROTOCOL_XROOTD_MAP,
    SERVER_ROOT_URI,
)


class DownloaderHttpRequests:
    """Downloader class for managing download related utilities with requests downloader engine."""

    def __init__(self, path, file_location, mode, file_size_offline):
        """Initialise class instance."""
        self.kb = 1024
        self.path = path
        self.mode = mode
        self.file_location = file_location
        self.file_name = self.file_location.split("/")[-1]
        self.file_dest = self.path + "/" + self.file_name
        self.file_size_offline = file_size_offline if file_size_offline else 0

    def show_download_progress(self, download_t=None, download_d=None):
        """Show download progress of a file."""
        display_message(
            msg_type="progress",
            msg="Progress: {}/{} KiB ({}%)\r".format(
                str(int(download_d / self.kb)),
                str(int(download_t / self.kb)),
                str(int(download_d / download_t * 100) if download_t > 0 else 0),
            ),
        )
        sys.stdout.flush()

    def file_downloader(self):
        """Download single file with requests."""
        headers = {}
        if self.file_size_offline:
            headers["Range"] = "bytes={}-".format(self.file_size_offline)
        response = requests.get(self.file_location, headers=headers, stream=True)
        total_size = int(response.headers.get("content-length", 0))
        with open(self.file_dest, self.mode) as f:
            display_message(
                msg_type="note",
                msg="File: ./{}/{}".format(
                    self.path,
                    self.file_name,
                ),
            )
            downloaded = self.file_size_offline
            total_size = total_size + self.file_size_offline
            for data in response.iter_content(chunk_size=1024):
                downloaded += len(data)
                try:
                    f.write(data)
                except Exception:
                    display_message(
                        msg_type="error",
                        msg="Download error occured. Please try again.",
                    )
                    sys.exit(1)
                self.show_download_progress(
                    download_t=total_size, download_d=downloaded
                )
                headers = {}


class DownloaderHttpPycurl:
    """Downloader class for managing download related utilities with pycurl downloader engine."""

    def __init__(self, path, file_location, mode, file_size_offline):
        """Initialise class instance."""
        self.kb = 1024
        self.path = path
        self.mode = mode
        self.file_location = file_location
        self.file_name = self.file_location.split("/")[-1]
        self.file_dest = self.path + "/" + self.file_name
        self.file_size_offline = file_size_offline if file_size_offline else 0

    def show_download_progress(
        self, download_t=None, download_d=None, upload_t=None, upload_d=None
    ):
        """Show download progress of a file."""
        download_t = download_t + self.file_size_offline
        download_d = download_d + self.file_size_offline
        display_message(
            msg_type="progress",
            msg="Progress: {}/{} KiB ({}%)\r".format(
                str(int(download_d / self.kb)),
                str(int(download_t / self.kb)),
                str(int(download_d / download_t * 100) if download_t > 0 else 0),
            ),
        )
        sys.stdout.flush()

    def file_downloader(self):
        """Download single file with pycurl."""
        c = pycurl.Curl()
        c.setopt(c.URL, self.file_location)
        if self.mode == "ab":
            c.setopt(c.RESUME_FROM, self.file_size_offline)
        with open(self.file_dest, self.mode) as f:
            display_message(
                msg_type="note",
                msg="File: ./{}/{}".format(
                    self.path,
                    self.file_name,
                ),
            )
            c.setopt(c.WRITEDATA, f)
            c.setopt(c.NOPROGRESS, False)
            c.setopt(c.XFERINFOFUNCTION, self.show_download_progress)
            try:
                c.perform()
            except Exception:
                display_message(
                    msg_type="error",
                    msg="Download error occured. Please try again.",
                )
                sys.exit(1)
            c.close()


class DownloaderXrootd:
    """Downloader class for managing download related utilities with xrootd downloader engine."""

    def __init__(self, path, file_location, mode):
        """Initialise class instance."""
        self.path = path
        self.mode = mode
        self.file_location = file_location
        self.file_name = self.file_location.split("/")[-1]
        self.file_dest = self.path + "/" + self.file_name
        self.file_src = self.file_location.split("root://eospublic.cern.ch/")[-1]

    def show_download_progress(self):
        """Show download progress of a file."""
        return

    def file_downloader(self):
        """Download single file with XRootD."""
        try:
            display_message(
                msg_type="note",
                msg="File: ./{}/{}".format(
                    self.path,
                    self.file_name,
                ),
            )
            process = xrootdclient.CopyProcess()
            process.add_job(
                SERVER_ROOT_URI + self.file_src, os.getcwd() + os.sep + self.file_dest
            )
            process.prepare()
            process.run()
        except Exception:
            display_message(
                msg_type="error", msg="Download error occured. Please try again."
            )


def check_error(
    path=None, file_location=None, protocol=None, retry_limit=None, retry_sleep=None
):
    """Return True if the file size and checksum does not matches with download error page.

    :param path: Directory where file is downloaded
    :param file_location: Remote location of a file
    :param protocol: Protocol to be used for downloading a file
    :param retry_limit: Number of retries to be made for downloading a file.
    :param retry_sleep: Time of sleep before every retry.
    :type path: str
    :type file_location: str
    :type protocol: str
    :type retry_limit: int
    :type retry_sleep: int

    :return: True if the file size and checksum does not matches with download error page.
    :rtype: Boolean
    """
    file_name = file_location.split("/")[-1]
    file_dest = path + "/" + file_name
    downloaded_file = {
        "size": os.path.getsize(file_dest),
        "checksum": get_file_checksum(file_dest),
    }
    if (
        DOWNLOAD_ERROR_PAGE["size"] == downloaded_file["size"]
        and DOWNLOAD_ERROR_PAGE["checksum"] == downloaded_file["checksum"]
    ):
        for _retry in range(0, retry_limit + 1):
            if _retry == retry_limit:
                display_message(msg_type="error", msg="Number of retries exceeded.")
                sys.exit(1)
            display_message(
                msg_type="note", msg="Retrying {}/{}".format(_retry + 1, retry_limit)
            )
            time.sleep(retry_sleep)
            download_single_file(
                path=path, file_location=file_location, protocol=protocol
            )
            downloaded_file = {
                "size": os.path.getsize(file_dest),
                "checksum": get_file_checksum(file_dest),
            }
            error = (
                DOWNLOAD_ERROR_PAGE["size"] == downloaded_file["size"]
                and DOWNLOAD_ERROR_PAGE["checksum"] == downloaded_file["checksum"]
            )
            if not error:
                return True


def downloader_file_checker(file_location, file_dest):
    """Return False if file is not present in the directory else True.

    :param file_location: Remote location of a file
    :param file_dest: Expected local destination path of a file
    :type file_location: str
    :type file_dest: str

    :return: False if file is not present in the directory else True
    :rtype: Boolean
    """
    try:
        response = requests.head(file_location)
        file_size_online = int(response.headers.get("content-length", 0))
    except Exception:
        display_message(
            msg_type="error",
            msg="Download error occured. Please try again.",
        )
    if os.path.isfile(file_dest):
        file_size_offline = os.path.getsize(file_dest)
        return file_size_online != file_size_offline
    return False


def download_single_file(
    path=None, file_location=None, protocol=None, download_engine=None
):
    """Download a single file.

    :param path: Directory where file is downloaded
    :param file_location: Remote location of a file
    :param protocol: Protocol to be used for downloading a file
    :param download_engine: Library to be used in downloading files
    :type path: str
    :type file_location: str
    :type protocol: str
    :type download_engine: str

    :return: None
    :rtype: None
    """
    file_name = file_location.split("/")[-1]
    file_dest = path + "/" + file_name
    download_engine_map = {
        "requests": requests_available,
        "pycurl": pycurl_available,
        "xrootd": xrootd_available,
    }
    if download_engine:
        if not download_engine_map.get(download_engine):
            display_message(
                msg_type="error",
                msg="{} is not installed on system. Please install it.".format(
                    download_engine
                ),
            )
            sys.exit(1)
    if protocol in ["http", "https"]:
        if download_engine not in DOWNLOAD_ENGINE_PROTOCOL_HTTP_MAP:
            display_message(
                msg_type="error",
                msg="{} is not compatible with {} protocol. Please use requests or pycurl download engine.".format(
                    download_engine,
                    protocol,
                ),
            )
            sys.exit(1)
        file_download_incomplete = downloader_file_checker(file_location, file_dest)
        if file_download_incomplete:
            file_size_offline = os.path.getsize(file_dest)
            mode = "ab"
            display_message(
                msg_type="note",
                msg="File {} is incomplete. Resuming download.".format(
                    file_name,
                ),
            )
        else:
            file_size_offline = None
            mode = "wb"
        if download_engine == "requests":
            downloader = DownloaderHttpRequests(
                path, file_location, mode, file_size_offline
            )
            downloader.file_downloader()
        elif download_engine == "pycurl":
            downloader = DownloaderHttpPycurl(
                path, file_location, mode, file_size_offline
            )
            downloader.file_downloader()
        print()
    elif protocol == "xrootd":
        if download_engine not in DOWNLOAD_ENGINE_PROTOCOL_XROOTD_MAP:
            display_message(
                msg_type="error",
                msg="{} is not compatible with {} protocol. Please use xrootd engine.".format(
                    download_engine,
                    protocol,
                ),
            )
            sys.exit(1)
        mode = "wb"
        downloader = DownloaderXrootd(path, file_location, mode)
        downloader.file_downloader()
    return


def get_download_files_by_name(names=None, file_locations=None):
    """Return the files filtered by file names.

    :param names: List of file names to be filtered
    :param file_locations: List of remote file locations
    :type names: list
    :type file_locations: list

    :return: List of file locations to be downloaded
    :rtype: list
    """
    download_file_locations = []
    for name in names:
        for file_location in file_locations:
            file_name = file_location.split("/")[-1]
            if file_name == name:
                download_file_locations.append(file_location)
    return download_file_locations


def get_download_files_by_regexp(regexp=None, file_locations=None, filtered_files=None):
    """Return the list of files filtered by a regular expression.

    :param regexp: Regexp string for filtering of file locations
    :param file_locations: List of remote file locations
    :param filtered_files: List of file locations filtered by previous filters(if any).
    :type regexp: str
    :type file_locations: list
    :type filtered_files: list

    :return: List of file locations to be downloaded
    :rtype: list
    """
    file_locations = filtered_files if filtered_files else file_locations
    download_file_locations = []
    for file_location in file_locations:
        file_name = file_location.split("/")[-1]
        if re.search(regexp, file_name):
            download_file_locations.append(file_location)
    return download_file_locations


def get_download_files_by_range(ranges=None, file_locations=None, filtered_files=None):
    """Return the list of files filtered by a range of files.

    :param ranges: List of ranges for filtering of files
    :param file_locations: List of remote file locations
    :param filtered_files: List of file locations filtered by previous filters(if any).
    :type ranges: list
    :type file_locations: list
    :type filtered_files: list

    :return: List of file locations to be downloaded
    :rtype: list
    """
    file_locations = filtered_files if filtered_files else file_locations
    download_file_locations = []
    for range in ranges:
        validate_range(range=range, count=len(file_locations))
        file_range = range.split("-")
        _range_file_locations = file_locations[
            int(file_range[0]) - 1 : int(file_range[-1])
        ]
        for file in _range_file_locations:
            download_file_locations.append(file)
    return download_file_locations



# -*- coding: utf-8 -*-
#
# This file is part of cernopendata-client.
#
# Copyright (C) 2020, 2021 CERN.
#
# cernopendata-client is free software; you can redistribute it and/or modify
# it under the terms of the GPLv3 license; see LICENSE file for more details.

"""cernopendata-client cli command download-files test."""

import os
import pytest

from click.testing import CliRunner
from cernopendata_client.cli import download_files
from cernopendata_client.config import SERVER_HTTPS_URI


def test_dry_run_from_recid():
    """Test `download-files --recid --dry-run` command."""
    test_dry_run = CliRunner()
    test_result = test_dry_run.invoke(download_files, ["--recid", 3005, "--dry-run"])
    assert test_result.exit_code == 0
    assert "0d0714743f0204ed3c0144941e6ce248.configFile.py" in test_result.output


def test_dry_run_from_recid_wrong():
    """Test `download-files --recid --dry-run` command for wrong values."""
    test_dry_run = CliRunner()
    test_result = test_dry_run.invoke(download_files, ["--recid", 0])
    assert test_result.exit_code == 2


def test_dry_run_from_doi():
    """Test `download-files --doi --dry-run` command."""
    test_dry_run = CliRunner()
    test_result = test_dry_run.invoke(
        download_files,
        ["--doi", "10.7483/OPENDATA.CMS.A342.9982", "--no-expand", "--dry-run"],
    )
    assert test_result.exit_code == 0
    assert (
        "CMS_Run2010B_BTau_AOD_Apr21ReReco-v1_0001_file_index.json"
        in test_result.output
    )


def test_dry_run_from_doi_wrong():
    """Test `download-files --doi --dry-run` command for wrong values."""
    test_dry_run = CliRunner()
    test_result = test_dry_run.invoke(
        download_files, ["--doi", "NONEXISTING", "--no-expand", "--dry-run"]
    )
    assert test_result.exit_code == 2


def test_download_files_http_pycurl():
    """Test download_files() command with http protocol using pycurl."""
    pycurl = pytest.importorskip("pycurl")  # noqa: F841
    test_file = "3005/0d0714743f0204ed3c0144941e6ce248.configFile.py"
    if os.path.isfile(test_file):
        os.remove(test_file)
    test_download_files = CliRunner()
    test_result = test_download_files.invoke(download_files, ["--recid", 3005])
    assert test_result.exit_code == 0
    assert os.path.isfile(test_file) is True
    assert os.path.getsize(test_file) == 3644
    assert test_result.output.endswith("\n==> Success!\n")
    if os.path.isfile(test_file):
        os.remove(test_file)


def test_download_files_http_requests(mocker):
    """Test download_files() command with http protocol using requests."""
    mocker.patch("cernopendata_client.downloader.pycurl_available", False)
    test_file = "3005/0d0714743f0204ed3c0144941e6ce248.configFile.py"
    if os.path.isfile(test_file):
        os.remove(test_file)
    test_download_files = CliRunner()
    test_result = test_download_files.invoke(download_files, ["--recid", 3005])
    assert test_result.exit_code == 0
    assert os.path.isfile(test_file) is True
    assert os.path.getsize(test_file) == 3644
    assert test_result.output.endswith("\n==> Success!\n")
    if os.path.isfile(test_file):
        os.remove(test_file)


def test_download_files_https_pycurl():
    """Test download_files() command with https protocol using pycurl."""
    pycurl = pytest.importorskip("pycurl")  # noqa: F841
    test_file = "3005/0d0714743f0204ed3c0144941e6ce248.configFile.py"
    if os.path.isfile(test_file):
        os.remove(test_file)
    test_download_files = CliRunner()
    test_result = test_download_files.invoke(
        download_files, ["--recid", 3005, "--server", SERVER_HTTPS_URI]
    )
    assert test_result.exit_code == 0
    assert os.path.isfile(test_file) is True
    assert os.path.getsize(test_file) == 3644
    assert test_result.output.endswith("\n==> Success!\n")
    if os.path.isfile(test_file):
        os.remove(test_file)


def test_download_files_https_requests(mocker):
    """Test download_files() command with https protocol using requests."""
    mocker.patch("cernopendata_client.downloader.pycurl_available", False)
    test_file = "3005/0d0714743f0204ed3c0144941e6ce248.configFile.py"
    if os.path.isfile(test_file):
        os.remove(test_file)
    test_download_files = CliRunner()
    test_result = test_download_files.invoke(
        download_files, ["--recid", 3005, "--server", SERVER_HTTPS_URI]
    )
    assert test_result.exit_code == 0
    assert os.path.isfile(test_file) is True
    assert os.path.getsize(test_file) == 3644
    assert test_result.output.endswith("\n==> Success!\n")
    if os.path.isfile(test_file):
        os.remove(test_file)


def test_download_files_download_engine(mocker):
    """Test download_files() command with download-engine option."""
    mocker.patch("cernopendata_client.downloader.pycurl_available", False)
    test_file = "3005/0d0714743f0204ed3c0144941e6ce248.configFile.py"
    if os.path.isfile(test_file):
        os.remove(test_file)
    test_download_files = CliRunner()
    test_result = test_download_files.invoke(
        download_files, ["--recid", 3005, "--download-engine", "requests"]
    )
    assert test_result.exit_code == 0
    assert os.path.isfile(test_file) is True
    assert os.path.getsize(test_file) == 3644
    assert test_result.output.endswith("\n==> Success!\n")
    if os.path.isfile(test_file):
        os.remove(test_file)


def test_download_files_download_engine_wrong_protocol_combination_one():
    """Test download_files() command with download-engine option and wrong protocol."""
    xrootd = pytest.importorskip("XRootD")  # noqa: F841
    test_download_files = CliRunner()
    test_result = test_download_files.invoke(
        download_files,
        ["--recid", 3005, "--download-engine", "requests", "--protocol", "xrootd"],
    )
    assert test_result.exit_code == 1
    assert "requests is not compatible with xrootd" in test_result.output


def test_download_files_download_engine_wrong_protocol_combination_two():
    """Test download_files() command with download-engine option and wrong protocol."""
    xrootd = pytest.importorskip("XRootD")  # noqa: F841
    test_download_files = CliRunner()
    test_result = test_download_files.invoke(
        download_files,
        ["--recid", 3005, "--download-engine", "xrootd", "--protocol", "http"],
    )
    assert test_result.exit_code == 1
    assert "xrootd is not compatible with http" in test_result.output


def test_download_files_root():
    """Test download_files() command with xrootd protocol."""
    xrootd = pytest.importorskip("XRootD")  # noqa: F841
    test_file = "3005/0d0714743f0204ed3c0144941e6ce248.configFile.py"
    if os.path.isfile(test_file):
        os.remove(test_file)
    test_download_files = CliRunner()
    test_result = test_download_files.invoke(
        download_files, ["--recid", 3005, "--protocol", "xrootd"]
    )
    assert test_result.exit_code == 0
    assert os.path.isfile(test_file) is True
    assert os.path.getsize(test_file) == 3644
    assert test_result.output.endswith("\n==> Success!\n")
    if os.path.isfile(test_file):
        os.remove(test_file)


def test_download_files_root_wrong(mocker):
    """Test download_files() command with xrootd protocol without xrootd."""
    mocker.patch("cernopendata_client.downloader.xrootd_available", False)
    test_download_files = CliRunner()
    test_result = test_download_files.invoke(
        download_files, ["--recid", 3005, "--protocol", "xrootd"]
    )
    assert test_result.exit_code == 1
    assert "xrootd is not installed on system" in test_result.output


def test_download_files_with_verify():
    """Test download_files() --verify command."""
    test_file = "3005/0d0714743f0204ed3c0144941e6ce248.configFile.py"
    if os.path.isfile(test_file):
        os.remove(test_file)
    test_download_files = CliRunner()
    test_result = test_download_files.invoke(
        download_files, ["--recid", 3005, "--verify"]
    )
    assert test_result.exit_code == 0
    assert os.path.isfile(test_file) is True
    assert os.path.getsize(test_file) == 3644
    assert test_result.output.endswith("\n==> Success!\n")
    if os.path.isfile(test_file):
        os.remove(test_file)


def test_download_files_empty_value():
    """Test download_files() command with empty value."""
    test_download_files_empty_value = CliRunner()
    test_result = test_download_files_empty_value.invoke(download_files)
    assert test_result.exit_code == 1
    assert "Please provide at least one of following arguments" in test_result.output


def test_download_files_wrong_value():
    """Test download_files() command with wrong value."""
    test_download_files_empty_value = CliRunner()
    test_result = test_download_files_empty_value.invoke(
        download_files,
        ["--recid", 5500, "--server", "foo"],
    )
    assert test_result.exit_code == 2
    assert "Invalid value for --server" in test_result.output


def test_download_files_filter_name():
    """Test download_files() command with --filter-name <name>."""
    test_file = "5500/BuildFile.xml"
    if os.path.isfile(test_file):
        os.remove(test_file)
    test_download_files_filter = CliRunner()
    test_result_name = test_download_files_filter.invoke(
        download_files,
        [
            "--recid",
            5500,
            "--filter-name",
            "BuildFile.xml",
        ],
    )
    assert test_result_name.exit_code == 0
    assert os.path.isfile(test_file) is True
    assert test_result_name.output.endswith("\n==> Success!\n")
    if os.path.isfile(test_file):
        os.remove(test_file)


def test_download_files_filter_name_multiple_values():
    """Test download_files() command with --filter-name <name1>,<name2>."""
    test_files = ["5500/BuildFile.xml", "5500/List_indexfile.txt"]
    for file_location in test_files:
        if os.path.isfile(file_location):
            os.remove(file_location)
    test_download_files_filter = CliRunner()
    test_result_name = test_download_files_filter.invoke(
        download_files,
        [
            "--recid",
            5500,
            "--filter-name",
            "BuildFile.xml,List_indexfile.txt",
        ],
    )
    assert test_result_name.exit_code == 0
    for file_location in test_files:
        if os.path.isfile(file_location):
            os.remove(file_location)
        else:
            pytest.fail("{} not downloaded".format(file_location))
    assert test_result_name.output.endswith("\n==> Success!\n")


def test_download_files_filter_name_wrong():
    """Test download_files() command with --filter-name <name> containing wrong value."""
    test_download_files_filter = CliRunner()
    test_result_name = test_download_files_filter.invoke(
        download_files, ["--recid", 5500, "--filter-name", "test_name.py"]
    )
    assert test_result_name.exit_code == 1
    assert "No files matching the filters" in test_result_name.output


def test_download_files_filter_regexp_single_file():
    """Test download_files() command with --filter-regexp."""
    test_file = "3005/0d0714743f0204ed3c0144941e6ce248.configFile.py"
    if os.path.isfile(test_file):
        os.remove(test_file)
    test_download_files_filter = CliRunner()
    test_result_regexp = test_download_files_filter.invoke(
        download_files, ["--recid", 3005, "--filter-regexp", "py$"]
    )
    assert test_result_regexp.exit_code == 0
    assert test_result_regexp.output.endswith("\n==> Success!\n")
    if os.path.isfile(test_file):
        os.remove(test_file)
    else:
        pytest.fail("{} not downloaded".format(test_file))


def test_download_files_filter_regexp_multiple_files():
    """Test download_files() command with --filter-regexp."""
    test_files = [
        "5500/HiggsDemoAnalyzer.cc",
        "5500/M4Lnormdatall.cc",
        "5500/M4Lnormdatall_lvl3.cc",
    ]
    for file_location in test_files:
        if os.path.isfile(file_location):
            os.remove(file_location)
    test_download_files_filter = CliRunner()
    test_result_regexp = test_download_files_filter.invoke(
        download_files, ["--recid", 5500, "--filter-regexp", ".cc$"]
    )
    assert test_result_regexp.exit_code == 0
    for file_location in test_files:
        if os.path.isfile(file_location):
            os.remove(file_location)
        else:
            pytest.fail("{} not downloaded".format(file_location))
    assert test_result_regexp.output.endswith("\n==> Success!\n")


def test_download_files_filter_regexp_wrong():
    """Test download_files() command with --filter-regexp containing wrong value."""
    test_download_files_filter = CliRunner()
    test_result_regexp_three = test_download_files_filter.invoke(
        download_files, ["--recid", 5500, "--filter-regexp", "wontmatchanything"]
    )
    assert test_result_regexp_three.exit_code == 1
    assert "No files matching the filters" in test_result_regexp_three.output


def test_download_files_filter_range():
    """Test download_files() command with --filter-range <range>."""
    test_files = [
        "5500/BuildFile.xml",
        "5500/HiggsDemoAnalyzer.cc",
        "5500/List_indexfile.txt",
        "5500/M4Lnormdatall.cc",
    ]
    for file_location in test_files:
        if os.path.isfile(file_location):
            os.remove(file_location)
    test_download_files_filter = CliRunner()
    test_result_range = test_download_files_filter.invoke(
        download_files, ["--recid", 5500, "--filter-range", "1-4"]
    )
    assert test_result_range.exit_code == 0
    for file_location in test_files:
        if os.path.isfile(file_location):
            os.remove(file_location)
        else:
            pytest.fail("{} not downloaded".format(file_location))
    assert test_result_range.output.endswith("\n==> Success!\n")


def test_download_files_filter_range_multiple_values():
    """Test download_files() command with --filter-range <range1>,<range2>."""
    test_files = [
        "5500/BuildFile.xml",
        "5500/HiggsDemoAnalyzer.cc",
        "5500/M4Lnormdatall_lvl3.cc",
        "5500/demoanalyzer_cfg_level3MC.py",
        "5500/demoanalyzer_cfg_level3data.py",
    ]
    for file_location in test_files:
        if os.path.isfile(file_location):
            os.remove(file_location)
    test_download_files_filter = CliRunner()
    test_result_range = test_download_files_filter.invoke(
        download_files, ["--recid", 5500, "--filter-range", "1-2,5-7"]
    )
    assert test_result_range.exit_code == 0
    for file_location in test_files:
        if os.path.isfile(file_location):
            os.remove(file_location)
        else:
            pytest.fail("{} not downloaded".format(file_location))
    assert test_result_range.output.endswith("\n==> Success!\n")


def test_download_files_filter_single_range_single_regexp():
    """Test download_files() command with --filter-regexp <regexp> --filter-range <range>."""
    test_files = [
        "5500/demoanalyzer_cfg_level3MC.py",
        "5500/demoanalyzer_cfg_level3data.py",
    ]
    for file_location in test_files:
        if os.path.isfile(file_location):
            os.remove(file_location)
    test_download_files_filter = CliRunner()
    test_result_range = test_download_files_filter.invoke(
        download_files,
        ["--recid", 5500, "--filter-regexp", "py", "--filter-range", "1-2"],
    )
    assert test_result_range.exit_code == 0
    for file_location in test_files:
        if os.path.isfile(file_location):
            os.remove(file_location)
        else:
            pytest.fail("{} not downloaded".format(file_location))
    assert test_result_range.output.endswith("\n==> Success!\n")


def test_download_files_filter_multiple_range_single_regexp():
    """Test download_files() command with --filter-regexp <regexp> --filter-range <range1>,<range2>."""
    test_files = [
        "5500/demoanalyzer_cfg_level3MC.py",
        "5500/demoanalyzer_cfg_level3data.py",
        "5500/demoanalyzer_cfg_level4data.py",
    ]
    for file_location in test_files:
        if os.path.isfile(file_location):
            os.remove(file_location)
    test_download_files_filter = CliRunner()
    test_result_range = test_download_files_filter.invoke(
        download_files,
        [
            "--recid",
            5500,
            "--filter-regexp",
            "py",
            "--filter-range",
            "1-2,4-4",
        ],
    )
    assert test_result_range.exit_code == 0
    for file_location in test_files:
        if os.path.isfile(file_location):
            os.remove(file_location)
        else:
            pytest.fail("{} not downloaded".format(file_location))
    assert test_result_range.output.endswith("\n==> Success!\n")



# -*- coding: utf-8 -*-
#
# This file is part of cernopendata-client.
#
# Copyright (C) 2020 CERN.
#
# cernopendata-client is free software; you can redistribute it and/or modify
# it under the terms of the GPLv3 license; see LICENSE file for more details.

"""cernopendata-client utils test."""

import click
import pytest

from cernopendata_client.utils import parse_parameters


def test_parse_parameters():
    """Test parse_parameters() method."""
    pytest.raises(SystemExit, parse_parameters, (9))
    assert parse_parameters(("test.py",)) == ["test.py"]
    assert parse_parameters(("2-4,9-12",)) == ["2-4", "9-12"]



# -*- coding: utf-8 -*-
#
# This file is part of REANA.
# Copyright (C) 2020 CERN.
#
# cernopendata-client is free software; you can redistribute it and/or modify
# it under the terms of the GPLv3 license; see LICENSE file for more details.

"""cernopendata-client output print configuration."""

import click

from .config import PRINTER_COLOUR_ERROR, PRINTER_COLOUR_INFO, PRINTER_COLOUR_NOTE


def display_message(msg_type=None, msg=None):
    """Display message in a similar style as run_command().

    :param msg_type: Type of message (info/note/error)
    :param msg: message to display
    :type msg_type: str
    :type msg: str
    """
    msg_color_map = {
        "info": PRINTER_COLOUR_INFO,
        "note": PRINTER_COLOUR_NOTE,
        "progress": PRINTER_COLOUR_NOTE,
        "error": PRINTER_COLOUR_ERROR,
    }
    msg_color = msg_color_map.get(msg_type, "")

    if msg_type == "info":
        click.secho("==> ", bold=True, nl=False, fg="{}".format(msg_color))
        click.secho("{}".format(msg), bold=True, nl=True)
    elif msg_type == "note":
        click.secho("  -> ", bold=False, nl=False, fg="{}".format(msg_color))
        click.secho("{}".format(msg), bold=False, nl=True)
    elif msg_type == "progress":
        click.secho("  -> ", bold=False, nl=False, fg="{}".format(msg_color))
        click.secho("{}".format(msg), bold=False, nl=False)
    elif msg_type == "error":
        click.secho(
            "==> {}: ".format(msg_type.upper()),
            bold=True,
            nl=False,
            fg="{}".format(msg_color),
        )
        click.secho("{}".format(msg), bold=False, nl=True)
    else:
        click.secho("{}".format(msg), nl=True)



# -*- coding: utf-8 -*-
#
# This file is part of cernopendata-client.
#
# Copyright (C) 2020 CERN.
#
# cernopendata-client is free software; you can redistribute it and/or modify
# it under the terms of the GPLv3 license; see LICENSE file for more details.

"""cernopendata-client file metadater test."""

from click.testing import CliRunner
from cernopendata_client.cli import get_metadata

from cernopendata_client.metadater import (
    filter_metadata,
    filter_matching_output,
)


def test_get_metadata_from_filter_fields_empty():
    """Test `get-metadata --recid --output-value --filter` command with empty values."""
    test_get_metadata = CliRunner()
    test_result = test_get_metadata.invoke(
        get_metadata,
        ["--recid", 1, "--output-value", "usage.links.description", "--filter", "url"],
    )
    assert test_result.exit_code == 1
    assert (
        "Invalid filter format. Use --filter some_field_name=some_value"
        in test_result.output
    )


def test_get_metadata_from_filter_metadata_one():
    """Test `get-metadata --recid --output-value --filter` command."""
    test_get_metadata = CliRunner()
    test_result = test_get_metadata.invoke(
        get_metadata,
        [
            "--recid",
            1,
            "--output-value",
            "usage.links.description",
            "--filter",
            "url=/docs/cms-getting-started-2010",
        ],
    )
    assert test_result.exit_code == 0
    assert "Getting started with CMS open data" in test_result.output


def test_get_metadata_from_filter_metadata_two():
    """Test `get-metadata --recid --output-value --filter` command."""
    test_get_metadata = CliRunner()
    test_result = test_get_metadata.invoke(
        get_metadata,
        [
            "--recid",
            451,
            "--output-value",
            "authors.name",
            "--filter",
            "affiliation=CERN",
            "--filter",
            "ccid=CCID-722528",
        ],
    )
    assert test_result.exit_code == 0
    assert "Plagge, Michael" in test_result.output


def test_get_metadata_from_filter_metadata_wrong_one():
    """Test `get-metadata --recid --output-value --filter` command for wrong values."""
    test_get_metadata = CliRunner()
    test_result = test_get_metadata.invoke(
        get_metadata,
        [
            "--recid",
            1,
            "--output-value",
            "usage.links.description",
            "--filter",
            "link=/docs/cms-getting-started-2010",
        ],
    )
    assert test_result.exit_code == 1
    assert "Field 'link' is not present in metadata" in test_result.output


def test_get_metadata_from_filter_metadata_wrong_two():
    """Test `get-metadata --recid --output-value --filter` command for wrong values."""
    test_get_metadata = CliRunner()
    test_result = test_get_metadata.invoke(
        get_metadata,
        [
            "--recid",
            1,
            "--output-value",
            "usage.links.description",
            "--filter",
            "url=/docs/cms-getting-started-20",
        ],
    )
    assert test_result.exit_code == 1
    assert (
        "No objects found with url=/docs/cms-getting-started-20" in test_result.output
    )



# -*- coding: utf-8 -*-
#
# This file is part of REANA.
# Copyright (C) 2020, 2021 CERN.
#
# cernopendata-client is free software; you can redistribute it and/or modify
# it under the terms of the GPLv3 license; see LICENSE file for more details.

"""cernopendata-client configuration."""


SERVER_HTTP_URI = "http://opendata.cern.ch"
"""Default CERN Open Data server to query over HTTP protocol."""

SERVER_HTTPS_URI = "https://opendata.cern.ch"
"""Default CERN Open Data server to query over HTTPS protocol."""

SERVER_ROOT_URI = "root://eospublic.cern.ch/"
"""Default CERN Open Data server to query over XRootD protocol."""

PRINTER_COLOUR_INFO = "green"
"""Default colour for info messages on terminal."""

PRINTER_COLOUR_NOTE = "blue"
"""Default colour for note messages on terminal."""

PRINTER_COLOUR_ERROR = "red"
"""Default colour for error messages on terminal."""

LIST_DIRECTORY_TIMEOUT = 60
"""Default timeout for list-directory command."""

DOWNLOAD_RETRY_LIMIT = 10
"""Default retries for getting a file from the server."""

DOWNLOAD_RETRY_SLEEP = 5
"""Sleep time in seconds before retrying downloads."""

DOWNLOAD_ERROR_PAGE = {"size": 3846, "checksum": "adler32:a82d5324"}
"""Error page info from the server."""

DOWNLOAD_ENGINE_PROTOCOL_HTTP_MAP = ["pycurl", "requests"]
"""Download engines compatible with HTTP protocol."""

DOWNLOAD_ENGINE_PROTOCOL_XROOTD_MAP = ["xrootd"]
"""Download engines compatible with xrootd protocol."""



# -*- coding: utf-8 -*-
#
# This file is part of cernopendata-client.
#
# Copyright (C) 2020, 2021 CERN.
#
# cernopendata-client is free software; you can redistribute it and/or modify
# it under the terms of the GPLv3 license; see LICENSE file for more details.

"""cernopendata-client file verifier tests."""

import os
import pytest
import subprocess

from click.testing import CliRunner
from cernopendata_client.cli import download_files, verify_files
from cernopendata_client.verifier import (
    get_file_size,
    get_file_checksum,
    get_file_info_local,
    verify_file_info,
)


def test_get_file_size():
    """Test get_file_size()."""
    afile = "./tests/test_version.py"
    assert get_file_size(afile) == 641


def test_get_file_checksum():
    """Test get_file_checksum()."""
    afile = "./tests/test_version.py"
    assert get_file_checksum(afile) == "adler32:fa91da1e"


def test_get_file_info_local_wrong_input():
    """Test get_file_info_local() for wrong inputs."""

    assert get_file_info_local(123456) == []


def test_get_file_info_local_good_input():
    """Test get_file_info_local() for good inputs."""

    # remove test file
    test_file = "3005/0d0714743f0204ed3c0144941e6ce248.configFile.py"
    if os.path.isfile(test_file):
        os.remove(test_file)

    # first download it
    test_download_files = CliRunner()
    test_result = test_download_files.invoke(download_files, ["--recid", 3005])
    assert test_result.exit_code == 0
    assert os.path.isfile(test_file) is True
    assert os.path.getsize(test_file) == 3644
    assert test_result.output.endswith("\n==> Success!\n")

    # now test get_file_info_local()
    test_result = get_file_info_local(3005)
    assert len(test_result) == 1
    assert test_result[0]["name"] == "0d0714743f0204ed3c0144941e6ce248.configFile.py"
    assert test_result[0]["size"] == 3644
    assert test_result[0]["checksum"] == "adler32:be83a186"

    # now test verifier
    test_verifier_files = CliRunner()
    test_result = test_verifier_files.invoke(verify_files, ["--recid", 3005])
    assert test_result.exit_code == 0
    assert test_result.output.endswith("\n==> Success!\n")

    # remove test file
    if os.path.isfile(test_file):
        os.remove(test_file)


def test_get_file_info_local_good_input_wrong_count():
    """Test get_file_info_local() for good inputs simulating wrong file count."""

    # remove test file
    test_file = "3005/0d0714743f0204ed3c0144941e6ce248.configFile.py"
    if os.path.isfile(test_file):
        os.remove(test_file)

    # first download it
    test_download_files = CliRunner()
    test_result = test_download_files.invoke(download_files, ["--recid", 3005])
    assert test_result.exit_code == 0
    assert os.path.isfile(test_file) is True
    assert os.path.getsize(test_file) == 3644
    assert test_result.output.endswith("\n==> Success!\n")

    # simulate getting more files
    cmd = "cp {} {}.extra".format(test_file, test_file)
    subprocess.check_output(cmd, shell=True)

    # now test verifier
    test_verifier_files = CliRunner()
    test_result = test_verifier_files.invoke(verify_files, ["--recid", 3005])
    assert test_result.exit_code == 1

    # remove test files
    if os.path.isfile(test_file + ".extra"):
        os.remove(test_file + ".extra")
    if os.path.isfile(test_file):
        os.remove(test_file)


def test_get_file_info_local_good_input_wrong_checksum():
    """Test get_file_info_local() for good inputs simulating wrong file checksum."""

    # remove test file
    test_file = "3005/0d0714743f0204ed3c0144941e6ce248.configFile.py"
    if os.path.isfile(test_file):
        os.remove(test_file)

    # first download it
    test_download_files = CliRunner()
    test_result = test_download_files.invoke(download_files, ["--recid", 3005])
    assert test_result.exit_code == 0
    assert os.path.isfile(test_file) is True
    assert os.path.getsize(test_file) == 3644
    assert test_result.output.endswith("\n==> Success!\n")

    # simulate checksum change
    cmd = "sed -i -e 's,a,b,g' {}".format(test_file)
    subprocess.check_output(cmd, shell=True)

    # now test verifier
    test_verifier_files = CliRunner()
    test_result = test_verifier_files.invoke(verify_files, ["--recid", 3005])
    assert test_result.exit_code == 1

    # remove test file
    if os.path.isfile(test_file):
        os.remove(test_file)


def test_get_file_info_local_good_input_wrong_size():
    """Test get_file_info_local() for good inputs simulating wrong file size."""

    # remove test file
    test_file = "3005/0d0714743f0204ed3c0144941e6ce248.configFile.py"
    if os.path.isfile(test_file):
        os.remove(test_file)

    # first download it
    test_download_files = CliRunner()
    test_result = test_download_files.invoke(download_files, ["--recid", 3005])
    assert test_result.exit_code == 0
    assert os.path.isfile(test_file) is True
    assert os.path.getsize(test_file) == 3644
    assert test_result.output.endswith("\n==> Success!\n")

    # simulate checksum change
    cmd = "sed -i -e 's,a,bbbbbb,g' {}".format(test_file)
    subprocess.check_output(cmd, shell=True)

    # now test verifier
    test_verifier_files = CliRunner()
    test_result = test_verifier_files.invoke(verify_files, ["--recid", 3005])
    assert test_result.exit_code == 1

    # remove test file
    if os.path.isfile(test_file):
        os.remove(test_file)


def test_verify_file_info_good_input():
    """Test verify_file_info() for good input."""

    # Remote test files info
    test_file_info_remote = [
        {
            "checksum": "adler32:ff63668a",
            "name": "BuildFile.xml",
            "size": 305,
            "uri": "http://opendata.cern.ch/eos/opendata/cms/software/HiggsExample20112012/BuildFile.xml",
        },
        {
            "checksum": "adler32:f205f068",
            "name": "HiggsDemoAnalyzer.cc",
            "size": 83761,
            "uri": "http://opendata.cern.ch/eos/opendata/cms/software/HiggsExample20112012/HiggsDemoAnalyzer.cc",
        },
        {
            "checksum": "adler32:46a907fc",
            "name": "List_indexfile.txt",
            "size": 1669,
            "uri": "http://opendata.cern.ch/eos/opendata/cms/software/HiggsExample20112012/List_indexfile.txt",
        },
        {
            "checksum": "adler32:af301992",
            "name": "M4Lnormdatall.cc",
            "size": 14943,
            "uri": "http://opendata.cern.ch/eos/opendata/cms/software/HiggsExample20112012/M4Lnormdatall.cc",
        },
    ]

    # Local test files info
    test_file_info_local = [
        {"name": "List_indexfile.txt", "size": 1669, "checksum": "adler32:46a907fc"},
        {"name": "M4Lnormdatall.cc", "size": 14943, "checksum": "adler32:af301992"},
        {"name": "BuildFile.xml", "size": 305, "checksum": "adler32:ff63668a"},
        {"name": "HiggsDemoAnalyzer.cc", "size": 83761, "checksum": "adler32:f205f068"},
    ]

    # Simulating function call
    assert verify_file_info(test_file_info_local, test_file_info_remote) is True


def test_verify_file_info_wrong_input():
    """Test verify_file_info() for wrong input."""

    # Remote test files info
    test_file_info_remote = [
        {
            "checksum": "adler32:ff63668a",
            "name": "BuildFile.xml",
            "size": 305,
            "uri": "http://opendata.cern.ch/eos/opendata/cms/software/HiggsExample20112012/BuildFile.xml",
        },
        {
            "checksum": "adler32:f205f068",
            "name": "HiggsDemoAnalyzer.cc",
            "size": 83761,
            "uri": "http://opendata.cern.ch/eos/opendata/cms/software/HiggsExample20112012/HiggsDemoAnalyzer.cc",
        },
        {
            "checksum": "adler32:46a907fc",
            "name": "List_indexfile.txt",
            "size": 1669,
            "uri": "http://opendata.cern.ch/eos/opendata/cms/software/HiggsExample20112012/List_indexfile.txt",
        },
        {
            "checksum": "adler32:af301992",
            "name": "M4Lnormdatall.cc",
            "size": 14943,
            "uri": "http://opendata.cern.ch/eos/opendata/cms/software/HiggsExample20112012/M4Lnormdatall.cc",
        },
    ]

    # Local test files info
    test_file_info_local = [
        {"name": "List_indexfile.txt", "size": 1669, "checksum": "adler32:46a907fc"},
        {"name": "M4Lnormdatall.cc", "size": 14943, "checksum": "adler32:af301992"},
        {"name": "BuildFile.xml", "size": 305, "checksum": "adler32:ff63668a"},
    ]

    # Simualting function call to exit for wrong input
    pytest.raises(
        SystemExit, verify_file_info, test_file_info_local, test_file_info_remote
    )


def test_verify_file_info_wrong_checksum():
    """Test verify_file_info() for wrong checksum."""

    # Remote test files info
    test_file_info_remote = [
        {
            "checksum": "adler32:ff63668a",
            "name": "BuildFile.xml",
            "size": 305,
            "uri": "http://opendata.cern.ch/eos/opendata/cms/software/HiggsExample20112012/BuildFile.xml",
        },
        {
            "checksum": "adler32:46a907fd",
            "name": "List_indexfile.txt",
            "size": 1669,
            "uri": "http://opendata.cern.ch/eos/opendata/cms/software/HiggsExample20112012/List_indexfile.txt",
        },
        {
            "checksum": "adler32:af301992",
            "name": "M4Lnormdatall.cc",
            "size": 14943,
            "uri": "http://opendata.cern.ch/eos/opendata/cms/software/HiggsExample20112012/M4Lnormdatall.cc",
        },
    ]

    # Local test files info
    test_file_info_local = [
        {"name": "List_indexfile.txt", "size": 1669, "checksum": "adler32:46a907fc"},
        {"name": "M4Lnormdatall.cc", "size": 14943, "checksum": "adler32:af301992"},
        {"name": "BuildFile.xml", "size": 305, "checksum": "adler32:ff63668a"},
    ]

    # Simualting function call to exit for wrong input
    pytest.raises(
        SystemExit, verify_file_info, test_file_info_local, test_file_info_remote
    )



# -*- coding: utf-8 -*-
#
# This file is part of cernopendata-client.
#
# Copyright (C) 2020, 2021 CERN.
#
# cernopendata-client is free software; you can redistribute it and/or modify
# it under the terms of the GPLv3 license; see LICENSE file for more details.

"""cernopendata-client cli command list-directory test."""

import pytest

from click.testing import CliRunner
from cernopendata_client.cli import list_directory


def test_non_recursive_list_directory():
    """Test `list_directory` command."""
    xrootd = pytest.importorskip("XRootD")  # noqa: F841
    test_non_recursive = CliRunner()
    test_result = test_non_recursive.invoke(
        list_directory, ["/eos/opendata/cms/validated-runs/Commissioning10"]
    )
    assert test_result.exit_code == 0
    assert "Commissioning10-May19ReReco_7TeV" in test_result.output


def test_recursive_list_directory():
    """Test `list_directory --recursive` command."""
    xrootd = pytest.importorskip("XRootD")  # noqa: F841
    test_recursive = CliRunner()
    test_result = test_recursive.invoke(
        list_directory, ["/eos/opendata/cms/Run2010B/BTau/AOD", "--recursive"]
    )
    assert test_result.exit_code == 0


def test_recursive_list_directory_timeout():
    """Test `list_directory --recursive` command with timeout."""
    xrootd = pytest.importorskip("XRootD")  # noqa: F841
    test_recursive = CliRunner()
    test_result = test_recursive.invoke(
        list_directory, ["/eos/opendata/cms", "--recursive", "--timeout", 5]
    )
    assert test_result.exit_code == 2
    assert "Command timed out." in test_result.output


def test_non_recursive_list_directory_wrong():
    """Test `list_directory` command with wrong path"""
    xrootd = pytest.importorskip("XRootD")  # noqa: F841
    test_non_recursive = CliRunner()
    test_result = test_non_recursive.invoke(list_directory, ["/eos/opendata/foobar"])
    assert test_result.exit_code == 1
    assert "Directory /eos/opendata/foobar does not exist." in test_result.output


def test_recursive_list_directory_wrong():
    """Test `list_directory --recursive` command with wrong path."""
    xrootd = pytest.importorskip("XRootD")  # noqa: F841
    test_recursive = CliRunner()
    test_result = test_recursive.invoke(
        list_directory, ["/eos/opendata/recursiveFoobar", "--recursive"]
    )
    assert (
        "Directory /eos/opendata/recursiveFoobar does not exist." in test_result.output
    )



# -*- coding: utf-8 -*-
#
# This file is part of cernopendata-client.
#
# Copyright (C) 2020 CERN.
#
# cernopendata-client is free software; you can redistribute it and/or modify
# it under the terms of the GPLv3 license; see LICENSE file for more details.

"""cernopendata-client file verifier related utilities."""

import os
import sys
import click
import zlib

from .printer import display_message


def get_file_size(afile):
    """Return the size of a file.

    :param afile: file name
    :type afile: str

    :return: Size of file
    :rtype: int
    """
    return os.path.getsize(afile)


def get_file_checksum(afile):
    """Return the ADLER32 checksum of a file.

    :param afile: file name
    :type afile: str

    :return: Adler32 checksum of file
    :rtype: str
    """
    return "adler32:" + hex(zlib.adler32(open(afile, "rb").read(), 1) & 0xFFFFFFFF)[2:]


def get_file_info_local(recid):
    """Return the local file information list for given record.

    :param recid: Record ID
    :type recid: str

    :return: Returns a list of dictionaries containing (checksum, name, size)
    for each file found downloaded in output directory matching recid.
    :rtype: list
    """
    file_info_local = []

    adir = str(recid)
    if not os.path.exists(adir):
        return file_info_local

    for afile in os.listdir(adir):
        file_info_local.append(
            {
                "name": afile,
                "size": get_file_size(adir + os.path.sep + afile),
                "checksum": get_file_checksum(adir + os.path.sep + afile),
            }
        )

    return file_info_local


def verify_file_info(file_info_local, file_info_remote):
    """Return True if local file info matches with the remote file info.

    :param file_info_local: List of files in local directory
    :param file_info_remote: List of files in remote server
    :type file_info_local: list
    :type file_info_remote: list

    :return: Bool if local file info matches with the remote file info.
    :rtype: Bool
    """
    for afile_info_remote in file_info_remote:
        afile_name = afile_info_remote["name"]
        afile_size = afile_info_remote["size"]
        afile_checksum = afile_info_remote["checksum"]
        bfile_size = 0
        bfile_checksum = ""
        for bfile_info_local in file_info_local:
            if bfile_info_local["name"] == afile_name:
                bfile_size = bfile_info_local["size"]
                bfile_checksum = bfile_info_local["checksum"]
                break
        display_message(
            msg_type="info",
            msg="Verifying file {}... ".format(afile_name),
        )
        display_message(
            msg_type="note",
            msg="Expected size {}, found {}".format(afile_size, bfile_size),
        )
        if afile_size != bfile_size:
            display_message(
                msg_type="error",
                msg="File size does not match.",
            )
            sys.exit(1)
        display_message(
            msg_type="note",
            msg="Expected checksum {}, found {}".format(afile_checksum, bfile_checksum),
        )
        if afile_checksum != bfile_checksum:
            display_message(msg_type="error", msg="File checksum does not match.")
            sys.exit(1)
    return True



# -*- coding: utf-8 -*-
#
# This file is part of cernopendata-client.
#
# Copyright (C) 2020 CERN.
#
# cernopendata-client is free software; you can redistribute it and/or modify
# it under the terms of the GPLv3 license; see LICENSE file for more details.

"""cernopendata-client cli command get-metadata test."""

from click.testing import CliRunner
from cernopendata_client.cli import get_metadata


def test_get_metadata_from_recid():
    """Test `get-metadata --recid` command."""
    test_get_metadata = CliRunner()
    test_result = test_get_metadata.invoke(get_metadata, ["--recid", 3005])
    assert test_result.exit_code == 0
    assert (
        '"title": "Configuration file for LHE step HIG-Summer11pLHE-00114_1_cfg.py"'
        in test_result.output
    )
    assert '"bucket": "1c1c9b67-ff5c-46bf-9e73-ef463d5bb1c6"' not in test_result.output
    assert (
        '"version_id": "d301cb45-86a2-4d8d-824e-f8ab7a716535"' not in test_result.output
    )


def test_get_metadata_from_recid_wrong():
    """Test `get-metadata --recid` command for wrong values."""
    test_get_metadata = CliRunner()
    test_result = test_get_metadata.invoke(get_metadata, ["--recid", 0])
    assert test_result.exit_code == 2


def test_get_metadata_from_doi():
    """Test `get-metadata --doi` command."""
    test_get_metadata = CliRunner()
    test_result = test_get_metadata.invoke(
        get_metadata, ["--doi", "10.7483/OPENDATA.CMS.A342.9982"]
    )
    assert test_result.exit_code == 0
    assert '"title": "/BTau/Run2010B-Apr21ReReco-v1/AOD"' in test_result.output


def test_get_metadata_from_doi_wrong():
    """Test `get-metadata --doi` command for wrong values."""
    test_get_metadata = CliRunner()
    test_result = test_get_metadata.invoke(get_metadata, ["--doi", "NONEXISTING"])
    assert test_result.exit_code == 2


def test_get_metadata_from_title_wrong():
    """Test `get-metadata --title` command for wrong values."""
    test_get_metadata = CliRunner()
    test_result = test_get_metadata.invoke(get_metadata, ["--title", "NONEXISTING"])
    assert test_result.exit_code == 2


def test_get_metadata_from_output_fields():
    """Test `get-metadata --recid --output-value` command."""
    test_get_metadata = CliRunner()
    test_result = test_get_metadata.invoke(
        get_metadata, ["--recid", 1, "--output-value", "system_details.global_tag"]
    )
    assert test_result.exit_code == 0
    assert "FT_R_42_V10A::All" in test_result.output


def test_get_metadata_from_output_fields_one():
    """Test `get-metadata --recid --output-value` command."""
    test_get_metadata = CliRunner()
    test_result = test_get_metadata.invoke(
        get_metadata, ["--recid", 1, "--output-value", "usage.links"]
    )
    assert test_result.exit_code == 0


def test_get_metadata_from_output_fields_two():
    """Test `get-metadata --recid --output-value` command."""
    test_get_metadata = CliRunner()
    test_result = test_get_metadata.invoke(
        get_metadata, ["--recid", 1, "--output-value", "usage.links.url"]
    )
    assert test_result.exit_code == 0


def test_get_metadata_from_output_fields_wrong():
    """Test `get-metadata --recid --output-value` command for wrong values."""
    test_get_metadata = CliRunner()
    test_result = test_get_metadata.invoke(
        get_metadata, ["--recid", 1, "--output-value", "title.global_tag"]
    )
    assert test_result.exit_code == 1
    assert "Field 'global_tag' is not present in metadata\n" in test_result.output


def test_get_metadata_empty_value():
    """Test get_metadata() command with empty value."""
    test_get_metadata_empty_value = CliRunner()
    test_result = test_get_metadata_empty_value.invoke(get_metadata)
    assert test_result.exit_code == 1
    assert "Please provide at least one of following arguments" in test_result.output


def test_get_metadata_wrong_value():
    """Test download_files() command with wrong value."""
    test_get_metadata_wrong_value = CliRunner()
    test_result = test_get_metadata_wrong_value.invoke(
        get_metadata,
        ["--recid", 5500, "--server", "foo"],
    )
    assert test_result.exit_code == 2
    assert "Invalid value for --server" in test_result.output



# -*- coding: utf-8 -*-
# This file is part of cernopendata-client.
#
# Copyright (C) 2020 CERN.
#
# cernopendata-client is free software; you can redistribute it and/or modify
# it under the terms of the GPLv3 license; see LICENSE file for more details.

"""cernopendata-client input validation methods."""

import click
import sys

from .printer import display_message


def validate_recid(recid=None):
    """Return True if record ID is valid, exit otherwise.

    :param recid: Record ID
    :type recid: int

    :return: Bool after verifying record ID
    :rtype: bool
    """
    if recid is None:
        display_message(
            msg_type="error",
            msg="You must supply a record ID number as an " "input using -r flag.",
        )
        sys.exit(1)
    if recid <= 0:
        display_message(
            msg_type="error",
            msg="Invalid value for {}: {} - Recid should be a positive integer".format(
                "--recid", recid
            ),
        )
        sys.exit(2)
    return True


def validate_server(server=None):
    """Return True if server uri is valid, exit otherwise.

    :param server: CERN Open Data server to query
    :type server: str

    :return: Bool after verfying server url
    :rtype: bool
    """
    server = server.split(":")
    if server[0] not in ["http", "https"]:
        display_message(
            msg_type="error",
            msg="Invalid value for {}: {} - Server should be a valid HTTP/HTTPS URI".format(
                "--server", server
            ),
        )
        sys.exit(2)
    return True


def validate_range(range=None, count=None):
    """Return True if range lies in total files count, exit otherwise.

    :param range: Range of files indexes
    :param count: Total files in a record
    :type range: str
    :type count: int

    :return: Bool after verifying range
    :rtype: bool
    """
    try:
        range_from, range_to = int(range.split("-")[0]), int(range.split("-")[-1])
    except Exception:
        display_message(
            msg_type="error",
            msg="Invalid value for {}: {} - Range should have start and end index(i-j)".format(
                "--filter-range", range
            ),
        )
        sys.exit(2)
    if range_from <= 0:
        display_message(
            msg_type="error",
            msg="Invalid value for {}: {} - Range should start from a positive integer".format(
                "--filter-range", range_from
            ),
        )
        sys.exit(2)
    if range_to > count:
        display_message(
            msg_type="error",
            msg="Invalid value for {}: {} - Range is too big. There are total {} files".format(
                "--filter-range", range, count
            ),
        )
        sys.exit(2)
    if range_to < range_from:
        display_message(
            msg_type="error",
            msg="Invalid value for {}: {} - Range is not valid".format(
                "--filter-range", range
            ),
        )
        sys.exit(2)
    return True


def validate_directory(directory=None):
    """Return True if directory path is correct, exit otherwise.

    :param directory: EOSPUBLIC path

    :return: Bool after verifying directory
    :rtype: bool
    """
    if sys.version_info[0] < 3:
        if isinstance(directory, unicode):  # noqa F821
            directory = directory.encode("utf-8")
    if isinstance(directory, str):
        if not directory.startswith("/eos/opendata/"):
            display_message(
                msg_type="error",
                msg="Invalid value for directory. {} is not valid EOSPUBLIC path".format(
                    directory
                ),
            )
            sys.exit(2)
    else:
        display_message(
            msg_type="error",
            msg="Invalid directory. {} is not valid string".format(directory),
        )
        sys.exit(2)
    return True


def validate_retry_limit(retry_limit=None):
    """Return True if retry_limit is valid, exit otherwise.

    :param retry_limit: Number of retries when downloading a file.

    :return: Bool after verifying retry_limit
    :rtype: bool
    """
    if retry_limit is None:
        display_message(
            msg_type="error",
            msg="You must supply a number input using --retry-limit flag.",
        )
        sys.exit(1)
    if retry_limit <= 0:
        display_message(
            msg_type="error",
            msg="Invalid value for {}: {} - Retry limit should be a positive integer".format(
                "--retry-limit", retry_limit
            ),
        )
        sys.exit(2)
    return True


def validate_retry_sleep(retry_sleep=None):
    """Return True if retry_sleep is valid, exit otherwise.

    :param retry_sleep: Sleep time in seconds before retrying downloads.

    :return: Bool after verifying retry_sleep
    :rtype: bool
    """
    if retry_sleep is None:
        display_message(
            msg_type="error",
            msg="You must supply a number input using --retry-sleep flag.",
        )
        sys.exit(1)
    if retry_sleep <= 0:
        display_message(
            msg_type="error",
            msg="Invalid value for {}: {} - Retry sleep should be a positive integer".format(
                "--retry-sleep", retry_sleep
            ),
        )
        sys.exit(2)
    return True



# -*- coding: utf-8 -*-
# This file is part of cernopendata-client.
#
# Copyright (C) 2019, 2020, 2022, 2024 CERN.
#
# cernopendata-client is free software; you can redistribute it and/or modify
# it under the terms of the GPLv3 license; see LICENSE file for more details.

"""Version information for cernopendata-client.

This file is imported by ``cernopendata-client.__init__`` and parsed by
``setup.py``.
"""

from __future__ import absolute_import, print_function

__version__ = "0.4.0"



# -*- coding: utf-8 -*-
# This file is part of cernopendata-client.
#
# Copyright (C) 2020, 2021 CERN.
#
# cernopendata-client is free software; you can redistribute it and/or modify
# it under the terms of the GPLv3 license; see LICENSE file for more details.

"""cernopendata-client EOSPUBLIC filesystem related utilities."""

from __future__ import print_function

import os
import sys
import datetime

from .config import SERVER_ROOT_URI
from .printer import display_message

try:
    from XRootD import client as xrootdclient
    from XRootD.client.flags import DirListFlags

    FS = xrootdclient.FileSystem(SERVER_ROOT_URI)
    xrootd_available = True
except ImportError:
    xrootd_available = False


def get_list_directory(path, recursive, timeout, time_start=None):
    """Return list of contents of a EOSPUBLIC Open Data directory.

    :param path: EOSPUBLIC path
    :param recursive: Iterate recursively into subdirectories?
    :param timeout: Timeout for list-directory command.
    :param time_start: Optionally, time When the recursive search started.
    :type path: str
    :type recursive: bool
    :type timeout: int
    :type time_start: datetime

    :return: List of files
    :rtype: list
    """
    if not xrootd_available:
        display_message(
            msg_type="error",
            msg="xrootd is required for this operation but it is not installed on your system.",
        )
        sys.exit(1)
    if not time_start:
        time_start = datetime.datetime.now()
    if (datetime.datetime.now() - time_start).seconds > timeout:
        display_message(
            msg_type="error",
            msg="Command timed out. Please increase timeout or provide more specific path.",
        )
        sys.exit(2)
    files = []
    try:
        status, listing = FS.dirlist(path, DirListFlags.STAT)
        for entry in listing:
            if entry.statinfo.flags == 19:  # entry is a directory
                files.append(path + os.sep + entry.name)
                if recursive:
                    files.extend(
                        get_list_directory(
                            path + os.sep + entry.name, recursive, timeout, time_start
                        )
                    )
            else:
                files.append(path + os.sep + entry.name)
        return files
    except Exception:
        display_message(
            msg_type="error",
            msg="Directory {} does not exist.".format(path),
        )
        sys.exit(1)
    return files



# -*- coding: utf-8 -*-
#
# This file is part of cernopendata-client.
#
# Copyright (C) 2020 CERN.
#
# cernopendata-client is free software; you can redistribute it and/or modify
# it under the terms of the GPLv3 license; see LICENSE file for more details.

"""cernopendata-client cli command get-file-locations test."""

from click.testing import CliRunner
from cernopendata_client.cli import get_file_locations
from cernopendata_client.config import SERVER_HTTPS_URI


def test_get_file_locations_from_recid():
    """Test `get-file-locations --recid` command."""
    test_get_file_locations = CliRunner()
    test_result = test_get_file_locations.invoke(get_file_locations, ["--recid", 3005])
    assert test_result.exit_code == 0
    assert "0d0714743f0204ed3c0144941e6ce248.configFile.py" in test_result.output


def test_get_file_locations_from_recid_without_files():
    """Test `get-file-locations --recid` command for recid without files."""
    test_get_file_locations = CliRunner()
    test_result = test_get_file_locations.invoke(get_file_locations, ["--recid", 550])
    assert test_result.exit_code == 0
    assert "" in test_result.output


def test_get_file_locations_from_recid_expand():
    """Test `get-file-locations --recid` command with expand."""
    test_get_file_locations = CliRunner()
    test_result = test_get_file_locations.invoke(get_file_locations, ["--recid", 282])
    assert test_result.exit_code == 0
    assert "2A227E10-C949-E311-B033-003048FEAF50.root" in test_result.output


def test_get_file_locations_from_recid_expand_verbose():
    """Test `get-file-locations --recid` command with expand and verbose."""
    test_get_file_locations = CliRunner()
    test_result = test_get_file_locations.invoke(
        get_file_locations, ["--recid", 282, "--verbose"]
    )
    assert test_result.exit_code == 0
    assert "2A227E10-C949-E311-B033-003048FEAF50.root" in test_result.output


def test_get_file_locations_from_recid_wrong():
    """Test `get-file-locations --recid` command for wrong values."""
    test_get_file_locations = CliRunner()
    test_result = test_get_file_locations.invoke(get_file_locations, ["--recid", 0])
    assert test_result.exit_code == 2


def test_get_file_locations_from_doi():
    """Test `get-file-locations --doi` command."""
    test_get_file_locations = CliRunner()
    test_result = test_get_file_locations.invoke(
        get_file_locations, ["--doi", "10.7483/OPENDATA.CMS.A342.9982", "--no-expand"]
    )
    assert test_result.exit_code == 0
    assert (
        "CMS_Run2010B_BTau_AOD_Apr21ReReco-v1_0001_file_index.json"
        in test_result.output
    )


def test_get_file_locations_from_doi_wrong():
    """Test `get-file-locations --doi` command for wrong values."""
    test_get_file_locations = CliRunner()
    test_result = test_get_file_locations.invoke(
        get_file_locations, ["--doi", "NONEXISTING", "--no-expand"]
    )
    assert test_result.exit_code == 2


def test_get_file_locations_with_verbose():
    """Test `get-file-locations --verbose` command."""
    test_get_file_locations = CliRunner()
    test_result = test_get_file_locations.invoke(
        get_file_locations, ["--recid", 5500, "--verbose"]
    )
    assert test_result.exit_code == 0
    assert "\t93152\tadler32:62e0c299\n" in test_result.output


def test_get_file_locations_with_https_server():
    """Test `get-file-locations --server` command for https server."""
    test_get_file_locations = CliRunner()
    test_result = test_get_file_locations.invoke(
        get_file_locations, ["--recid", 3005, "--server", SERVER_HTTPS_URI]
    )
    assert test_result.exit_code == 0
    assert "configFile.py" in test_result.output


def test_get_file_locations_with_https_server_xrootd_protocol():
    """Test `get-file-locations --server --protocol xrootd` command for https server."""
    test_get_file_locations = CliRunner()
    test_result = test_get_file_locations.invoke(
        get_file_locations,
        ["--recid", 3005, "--server", SERVER_HTTPS_URI, "--protocol", "xrootd"],
    )
    assert test_result.exit_code == 0
    assert "root://eospublic.cern.ch//eos/" in test_result.output



# -*- coding: utf-8 -*-
#
# This file is part of cernopendata-client.
#
# Copyright (C) 2020 CERN.
#
# cernopendata-client is free software; you can redistribute it and/or modify
# it under the terms of the GPLv3 license; see LICENSE file for more details.

"""cernopendata-client verify-files tests."""

import os
import pytest
from click.testing import CliRunner
from cernopendata_client.cli import download_files, verify_files
from cernopendata_client.config import SERVER_HTTPS_URI


def test_verify_files():
    """Test verify-files command."""

    # remove test file
    test_file = "3005/0d0714743f0204ed3c0144941e6ce248.configFile.py"
    if os.path.isfile(test_file):
        os.remove(test_file)

    # first download it
    test_download_files = CliRunner()
    test_result = test_download_files.invoke(download_files, ["--recid", 3005])
    assert test_result.exit_code == 0
    assert os.path.isfile(test_file) is True
    assert os.path.getsize(test_file) == 3644
    assert test_result.output.endswith("\n==> Success!\n")

    # now test verifier
    test_verify_files = CliRunner()
    test_result = test_verify_files.invoke(verify_files, ["--recid", 3005])
    assert test_result.exit_code == 0
    assert test_result.output.endswith("\n==> Success!\n")

    # remove test file
    if os.path.isfile(test_file):
        os.remove(test_file)


def test_verify_files_https_server():
    """Test verify-files command with https server."""

    # remove test file
    test_file = "3005/0d0714743f0204ed3c0144941e6ce248.configFile.py"
    if os.path.isfile(test_file):
        os.remove(test_file)

    # first download it
    test_download_files = CliRunner()
    test_result = test_download_files.invoke(
        download_files, ["--recid", 3005, "--server", SERVER_HTTPS_URI]
    )
    assert test_result.exit_code == 0
    assert os.path.isfile(test_file) is True
    assert os.path.getsize(test_file) == 3644
    assert test_result.output.endswith("\n==> Success!\n")

    # now test verifier
    test_verify_files = CliRunner()
    test_result = test_verify_files.invoke(
        verify_files, ["--recid", 3005, "--server", SERVER_HTTPS_URI]
    )
    assert test_result.exit_code == 0
    assert test_result.output.endswith("\n==> Success!\n")

    # remove test file
    if os.path.isfile(test_file):
        os.remove(test_file)


def test_verify_files_empty_value():
    """Test verify-files command with empty value."""
    test_verify_files_empty_value = CliRunner()
    test_result = test_verify_files_empty_value.invoke(verify_files)
    assert test_result.exit_code == 1
    assert "Please provide at least one of following arguments" in test_result.output


def test_verify_files_wrong_value():
    """Test verify-files command with wrong value."""
    test_verify_files_wrong_value = CliRunner()
    test_result = test_verify_files_wrong_value.invoke(
        verify_files,
        ["--recid", 5500, "--server", "foo"],
    )
    assert test_result.exit_code == 2
    assert "Invalid value for --server" in test_result.output


def test_verify_files_witout_download():
    """Test verify-files command."""
    test_file = "3005/0d0714743f0204ed3c0144941e6ce248.configFile.py"
    if os.path.isfile(test_file):
        os.remove(test_file)
    test_verify_files = CliRunner()
    test_result = test_verify_files.invoke(verify_files, ["--recid", 3005])
    assert test_result.exit_code == 1
    assert "No local files found for record 3005" in test_result.output



# -*- coding: utf-8 -*-
#
# This file is part of cernopendata-client.
#
# Copyright (C) 2019, 2020, 2021, 2022, 2023, 2024 CERN.
#
# cernopendata-client is free software; you can redistribute it and/or modify
# it under the terms of the GPLv3 license; see LICENSE file for more details.
"""cernopendata-client."""

import os
import re

from setuptools import setup

readme = open("README.rst").read()
history = open("CHANGES.rst").read()


extras_require = {
    "docs": [
        "jinja2<3.1.0",
        "Sphinx>=1.4.4",
        "sphinx-rtd-theme>=0.1.9",
        "sphinx-click>=2.5.0",
    ],
    "pycurl": ["pycurl>=7"],
    "tests": [
        'black>=19.10b0 ; python_version>="3"',
        "check-manifest>=0.25",
        "coverage>=4.0",
        "mock>=3.0",
        "pydocstyle>=1.0.0",
        "pytest-cache>=1.0",
        "pytest-cov>=1.8.0",
        "pytest>=2.8.0",
        'platformdirs<2.1 ; python_version=="3.6"',
        'pytest-mock>=2.0,<3.0 ; python_version=="2.7"',
        'pytest-mock>=3.0 ; python_version>="3"',
    ],
    "xrootd": [
        "xrootd>=4.12.2",
    ],
}

extras_require["all"] = []
for key, reqs in extras_require.items():
    if ":" == key[0]:
        continue
    extras_require["all"].extend(reqs)

install_requires = ["click>=7", "requests>=2"]

# Get the version string. Cannot be done with import!
with open(os.path.join("cernopendata_client", "version.py"), "rt") as f:
    version = re.search('__version__\s*=\s*"(?P<version>.*)"\n', f.read()).group(
        "version"
    )

setup(
    name="cernopendata-client",
    version=version,
    description=__doc__,
    long_description=readme + "\n\n" + history,
    author="CERN Open Data",
    author_email="opendata-team@cern.ch",
    packages=[
        "cernopendata_client",
    ],
    extras_require=extras_require,
    install_requires=install_requires,
    entry_points={
        "console_scripts": [
            "cernopendata-client = cernopendata_client.cli:cernopendata_client"
        ]
    },
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: System :: Archiving",
    ],
)



# -*- coding: utf-8 -*-
# This file is part of cernopendata-client.
#
# Copyright (C) 2020 CERN.
#
# cernopendata-client is free software; you can redistribute it and/or modify
# it under the terms of the GPLv3 license; see LICENSE file for more details.

"""cernopendata-client record search related utilities."""

from __future__ import print_function

import sys
import requests

from .config import SERVER_HTTP_URI, SERVER_ROOT_URI, SERVER_HTTPS_URI
from .printer import display_message

try:
    from urllib.parse import quote
except ImportError:
    # fallback for Python 2
    from urllib import quote


def verify_recid(server=None, recid=None):
    """Verify that recid corresponds to a valid Open Data record webpage.

    :param server: CERN Open Data server to query
    :param recid: Record ID
    :type server: str
    :type recid: int

    :return: Boolean after verifying the record ID
    :rtype: bool
    """
    input_record_url = server + "/record/" + str(recid)
    input_record_url_check = requests.get(input_record_url)

    if input_record_url_check.status_code == 200:
        base_record_id = str(recid)
        return base_record_id
    else:
        try:
            input_record_url_check.raise_for_status()
        except Exception:
            display_message(
                msg_type="error",
                msg="The record ID number you supplied is not valid.",
            )
            sys.exit(1)
        return False


def get_recid_api(server=None, base_record_id=None):
    """Return api for the record with given recid.

    :param server: CERN Open Data server to query
    :param base_record_id: Record ID
    :type server: str
    :type base_record_id: int

    :return: String of record api
    :rtype: str
    """
    record_api_url = server + "/api/records/" + base_record_id
    record_api = requests.get(record_api_url)
    try:
        record_api.raise_for_status()
    except Exception:
        display_message(
            msg_type="error",
            msg="The record ID number you supplied is not valid.",
        )
        sys.exit(1)
    return record_api


def get_recid(server=None, title=None, doi=None):
    """Return record ID by either title or doi.

    :param server: CERN Open Data server to query
    :param title: Record title
    :param doi: Digital Object Identifier of record
    :type server: str
    :type title: str
    :type doi: str

    :return: record ID
    :rtype: int
    """
    if title:
        name, value = "title", title
    elif doi:
        name, value = "doi", doi
    url = (
        server
        + "/api/records"
        + "?page=1&size=1&q={}:".format(name)
        + quote('"{}"'.format(value), safe="")
    )
    response = requests.get(url)
    response_json = response.json()
    try:
        response.raise_for_status()
    except Exception as e:
        display_message(
            msg_type="error",
            msg="Connection to server failed: \n reason: {}.".format(e),
        )
    if "hits" in response_json:
        hits_total = response_json["hits"]["total"]
        if hits_total < 1:
            display_message(
                msg_type="error",
                msg="Record with given {} does not exist.".format(name),
            )
            sys.exit(2)
        elif hits_total > 1:
            display_message(
                msg_type="error",
                msg="More than one record fit this {}."
                "This should not happen.".format(name),
            )
            sys.exit(3)
        elif hits_total == 1:
            return response_json["hits"]["hits"][0]["id"]


def get_record_as_json(server=None, recid=None, doi=None, title=None):
    """Return record content in json by its recid, doi or title.

    :param server: CERN Open Data server to query
    :param recid: Record ID
    :param title: Record title
    :param doi: Digital Object Identifier of record
    :type server: str
    :type recid: int
    :type title: str
    :type doi: str

    :return: record content in JSON
    :rtype: json(dict)
    """
    if recid:
        record_id = recid
    elif title:
        record_id = get_recid(server=server, title=title)
    elif doi:
        record_id = get_recid(server=server, doi=doi)
    else:
        display_message(
            msg_type="error",
            msg="Please provide at least one of following arguments: "
            "(recid, doi, title)",
        )
        sys.exit(1)

    record_id = verify_recid(server=server, recid=record_id)
    record_api = get_recid_api(server=server, base_record_id=record_id)
    record_json = record_api.json()
    if "_files" in record_json["metadata"]:
        del record_json["metadata"]["_files"]
    try:
        if record_json["metadata"]["files"]:
            for field in record_json["metadata"]["files"]:
                if "bucket" in field:
                    del field["bucket"]
                if "version_id" in field:
                    del field["version_id"]
    except KeyError:
        record_json["metadata"]["files"] = []
    return record_json


def get_files_list(
    server=None, record_json=None, protocol=None, expand=None, verbose=None
):
    """Return file list of a dataset by its recid, doi, or title.

    :param server: CERN Open Data server to query
    :param record_json: Record content in JSON
    :protocol: Protocol to be used in links [http,xrootd]
    :expand: Flag for expanding file indexes
    :verbose: Flag for showing size and checksum of file
    :type server: str
    :type record_json: json(dict)
    :type protocol: str
    :type expand: bool
    :type verbose: bool

    :return: List of files list
    :rtype: list
    """
    searcher_protocol = protocol
    if server != SERVER_HTTP_URI and searcher_protocol != "xrootd":
        searcher_protocol = server.split(":")[0]
    files_list = []
    for file_ in record_json["metadata"]["files"]:
        files_list.append((file_["uri"], file_["size"], file_["checksum"]))
    if expand:
        # let's unwind file indexes
        files_list_expanded = []
        for file_ in files_list:
            if file_[0].endswith("_file_index.json"):
                try:
                    url_file = "{}/record/{}/files/{}".format(
                        server, str(record_json["id"]), file_[0].split("/")[-1]
                    )
                    json_files = requests.get(url_file).json()
                except Exception:
                    display_message(
                        msg_type="error",
                        msg="Error occured while fetching file info. Please try again.",
                    )
                    sys.exit(1)
                for file_ in json_files:
                    files_list_expanded.append(
                        (
                            file_["uri"],
                            file_["size"],
                            file_["checksum"],
                        )
                    )
            elif file_[0].endswith("_file_index.txt"):
                pass
            else:
                files_list_expanded.append(file_)
        files_list = files_list_expanded
    if searcher_protocol == "http":
        files_list = [
            (file_[0].replace(SERVER_ROOT_URI, server), file_[1], file_[2])
            for file_ in files_list
        ]
    elif searcher_protocol == "https":
        files_list = [
            (
                file_[0].replace(SERVER_ROOT_URI, SERVER_HTTPS_URI),
                file_[1],
                file_[2],
            )
            for file_ in files_list
        ]
    return files_list


def get_file_info_remote(server, recid, protocol=None, filtered_files=None):
    """Return remote file information list for given record.

    :param server: CERN Open Data server to query
    :param recid: Record ID
    :param filtered_files: list of file locations after applying filters(if any)
    :type server: str
    :type recid: int
    :type filtered_files: list

    :return: Returns a list of dictionaries containing (checksum, name, size,
    uri) for each file in the record.  Note that 'name' property is
    not stored remotely, but is calculated in this function for
    convenience.
    :rtype: list
    """
    searcher_protocol = protocol
    file_info_remote = []
    if server != SERVER_HTTP_URI and searcher_protocol != "xrootd":
        searcher_protocol = server.split(":")[0]
    record_json = get_record_as_json(server=server, recid=recid)
    for file_info in record_json["metadata"]["files"]:
        file_checksum = file_info["checksum"]
        file_size = file_info["size"]
        file_uri = file_info["uri"]
        file_name = file_info["uri"].rsplit("/", 1)[1]
        if searcher_protocol == "http":
            file_uri = file_info["uri"].replace(SERVER_ROOT_URI, server)
        elif searcher_protocol == "https":
            file_uri = file_info["uri"].replace(SERVER_ROOT_URI, SERVER_HTTPS_URI)
        if not filtered_files or file_uri in filtered_files:
            file_info_remote.append(
                {
                    "checksum": file_checksum,
                    "name": file_name,
                    "size": file_size,
                    "uri": file_uri,
                }
            )
    return file_info_remote



# -*- coding: utf-8 -*-
# This file is part of cernopendata-client.
#
# Copyright (C) 2019 CERN.
#
# cernopendata-client is free software; you can redistribute it and/or modify
# it under the terms of the GPLv3 license; see LICENSE file for more details.
"""cernopendata-client version test."""

from cernopendata_client import __version__
from click.testing import CliRunner
from cernopendata_client.cli import version


def test_version():
    """Test version import."""
    test_version = CliRunner()
    test_result = test_version.invoke(version)
    assert test_result.exit_code == 0
    assert test_result.output == __version__ + "\n"



# -*- coding: utf-8 -*-
#
# This file is part of cernopendata-client.
#
# Copyright (C) 2019 CERN.
#
# cernopendata-client is free software; you can redistribute it and/or modify
# it under the terms of the GPLv3 license; see LICENSE file for more details.

"""cernopendata-client."""

from __future__ import absolute_import, print_function

from .version import __version__

__all__ = ("__version__",)



# -*- coding: utf-8 -*-
# This file is part of cernopendata-client.
#
# Copyright (C) 2019, 2020, 2021, 2023 CERN.
#
# cernopendata-client is free software; you can redistribute it and/or modify
# it under the terms of the GPLv3 license; see LICENSE file for more details.

"""cernopendata-client command line tool."""

import click
import json
import os
import requests
import sys
import re

from .searcher import (
    get_file_info_remote,
    get_files_list,
    get_recid,
    get_recid_api,
    get_record_as_json,
    verify_recid,
)
from .downloader import (
    check_error,
    download_single_file,
    get_download_files_by_name,
    get_download_files_by_range,
    get_download_files_by_regexp,
)
from .validator import (
    validate_range,
    validate_recid,
    validate_server,
    validate_directory,
    validate_retry_limit,
    validate_retry_sleep,
)
from .walker import get_list_directory
from .verifier import get_file_info_local, verify_file_info
from .metadater import filter_metadata, handle_error_message
from .config import (
    SERVER_HTTP_URI,
    LIST_DIRECTORY_TIMEOUT,
    DOWNLOAD_RETRY_LIMIT,
    DOWNLOAD_RETRY_SLEEP,
)
from .utils import parse_parameters
from .printer import display_message

from .version import __version__


@click.group()
def cernopendata_client():
    """Command-line client for interacting with CERN Open Data portal."""
    pass


@cernopendata_client.command()
def version():
    """Return cernopendata-client version.

    Examples: \n
    \t $ cernopendata-client version
    """
    display_message(msg=__version__)


@cernopendata_client.command()
@click.option("--recid", type=click.INT, help="Record ID")
@click.option("--doi", help="Digital Object Identifier")
@click.option("--title", help="Title of the record")
@click.option(
    "--output-value",
    is_flag=False,
    type=click.STRING,
    help="Output value of only desired metadata field [example=title]",
)
@click.option(
    "--server",
    default=SERVER_HTTP_URI,
    type=click.STRING,
    help="Which CERN Open Data server to query? [default={}]".format(SERVER_HTTP_URI),
)
@click.option(
    "--filter",
    "filters",
    multiple=True,
    help="Filter only certain output values matching filtering criteria. [Use --filter some_field_name=some_value]",
)
def get_metadata(server, recid, doi, title, output_value, filters):
    # noqa: D301
    """Get metadata content of a record.

    Select a CERN Open Data bibliographic record by a record ID, a
    DOI, or a title and return its metadata in the JSON format.

    Examples: \n
    \t $ cernopendata-client get-metadata --recid 1\n
    \t $ cernopendata-client get-metadata --recid 1 --output-value title\n
    \t $ cernopendata-client get-metadata --recid 329 --output-value authors.orcid --filter name="Rousseau, David"
    """
    validate_server(server)
    if recid is not None:
        validate_recid(recid)
    record_json = get_record_as_json(server, recid, doi, title)
    output_json = record_json["metadata"]
    if output_value:
        fields = output_value.split(".")
        wrong_field = True
        try:
            for field in fields:
                output_json = output_json[field]
            if filters:
                filter_metadata(field, filters, output_json)
                return
        except (KeyError, TypeError):
            try:
                if filters:
                    filter_metadata(field, filters, output_json)
                    wrong_field = False
                else:
                    for object in output_json:
                        if field in object:
                            wrong_field = False
                            display_message(msg=object[field])
            except (KeyError, TypeError):
                handle_error_message(field)
            if wrong_field:
                handle_error_message(field)
            return

        if isinstance(output_json, (dict, list)):
            display_message(msg=json.dumps(output_json, indent=4))
        else:  # print strings or numbers more simply
            display_message(msg=output_json)
    elif filters:
        display_message(
            msg_type="error",
            msg="--filter can only be used with --output-value",
        )
    else:
        display_message(msg=json.dumps(output_json, indent=4))


@cernopendata_client.command()
@click.option("--recid", type=click.INT, help="Record ID")
@click.option("--doi", help="Digital Object Identifier")
@click.option("--title", help="Record title")
@click.option(
    "--protocol",
    default="http",
    type=click.Choice(["http", "xrootd"]),
    help="Protocol to be used in links [http,xrootd]",
)
@click.option(
    "--expand/--no-expand", default=True, help="Expand file indexes? [default=yes]"
)
@click.option(
    "--server",
    default=SERVER_HTTP_URI,
    type=click.STRING,
    help="Which CERN Open Data server to query? [default={}]".format(SERVER_HTTP_URI),
)
@click.option(
    "--verbose",
    is_flag=True,
    default=False,
    help="Output also the file size (in the second column) and the file checksum (in the third column).",
)
def get_file_locations(server, recid, doi, title, protocol, expand, verbose):
    """Get a list of data file locations of a record.

    Select a CERN Open Data bibliographic record by a record ID, a
    DOI, or a title and return the list of data file locations
    belonging to this record.

    Examples: \n
    \t $ cernopendata-client get-file-locations --recid 5500\n
    \t $ cernopendata-client get-file-locations --recid 5500 --protocol xrootd\n
    \t $ cernopendata-client get-file-locations --recid 5500 --verbose
    """
    validate_server(server)
    if recid is not None:
        validate_recid(recid)
    record_json = get_record_as_json(server, recid, doi, title)
    file_locations = get_files_list(server, record_json, protocol, expand, verbose)
    if verbose:
        for file_ in file_locations:
            display_message(msg="{}\t{}\t{}".format(file_[0], file_[1], file_[2]))
    else:
        for file_ in file_locations:
            display_message(msg="{}".format(file_[0]))


@cernopendata_client.command()
@click.option("--recid", type=click.INT, help="Record ID")
@click.option("--doi", help="Digital Object Identifier")
@click.option("--title", help="Record title")
@click.option(
    "--protocol",
    default="http",
    type=click.Choice(["http", "xrootd"]),
    help="Protocol to be used in links [http,xrootd]",
)
@click.option(
    "--expand/--no-expand", default=True, help="Expand file indexes? [default=yes]"
)
@click.option(
    "--server",
    default=SERVER_HTTP_URI,
    type=click.STRING,
    help="Which CERN Open Data server to query? [default={}]".format(SERVER_HTTP_URI),
)
@click.option(
    "--dry-run",
    "dryrun",
    is_flag=True,
    default=False,
    help="Do not download anything, only print out data file locations to be downloaded",
)
@click.option(
    "--filter-name",
    "names",
    multiple=True,
    type=click.STRING,
    help="Download files matching exactly the file name",
)
@click.option(
    "--filter-regexp",
    "regexp",
    type=click.STRING,
    help="Download files matching the regular expression",
)
@click.option(
    "--filter-range",
    "ranges",
    multiple=True,
    type=click.STRING,
    help="Download files from a specified list range (i-j)",
)
@click.option(
    "--verify",
    "verify",
    is_flag=True,
    default=False,
    help="Verify downloaded data file integrity",
)
@click.option(
    "--retry-limit",
    "retry_limit",
    default=DOWNLOAD_RETRY_LIMIT,
    type=click.INT,
    help="Number of retries when downloading a file [default={}]".format(
        DOWNLOAD_RETRY_LIMIT
    ),
)
@click.option(
    "--retry-sleep",
    "retry_sleep",
    default=DOWNLOAD_RETRY_SLEEP,
    type=click.INT,
    help="Sleep time in seconds before retrying downloads [default={}]".format(
        DOWNLOAD_RETRY_SLEEP
    ),
)
@click.option(
    "--download-engine",
    "download_engine",
    type=click.Choice(["requests", "pycurl", "xrootd"]),
    help="Download engine to use when downloading files."
    "The available values are 'requests', 'pycurl', 'xrootd'."
    "[default=requests (for HTTP protocol), xrootd (for XRootD protocol)]",
)
def download_files(
    server,
    recid,
    doi,
    title,
    protocol,
    expand,
    names,
    regexp,
    ranges,
    dryrun,
    verify,
    retry_limit,
    retry_sleep,
    download_engine,
):
    """Download data files belonging to a record.

    Select a CERN Open Data bibliographic record by a record ID, a
    DOI, or a title and download data files belonging to this record.

    Examples: \n
    \t $ cernopendata-client download-files --recid 5500\n
    \t $ cernopendata-client download-files --recid 5500 --filter-name BuildFile.xml\n
    \t $ cernopendata-client download-files --recid 5500 --filter-regexp py$\n
    \t $ cernopendata-client download-files --recid 5500 --filter-range 1-4\n
    \t $ cernopendata-client download-files --recid 5500 --filter-range 1-2,5-7\n
    \t $ cernopendata-client download-files --recid 5500 --filter-regexp py --filter-range 1-2
    """
    validate_server(server)
    if recid is not None:
        validate_recid(recid)
    if retry_limit:
        validate_retry_limit(retry_limit=retry_limit)
    if retry_sleep:
        validate_retry_sleep(retry_sleep=retry_sleep)
    record_json = get_record_as_json(server, recid, doi, title)
    file_locations_info = get_files_list(server, record_json, protocol, expand)
    file_locations = [file_[0] for file_ in file_locations_info]
    download_file_locations = []

    if names:
        parsed_name_filters = parse_parameters(names)
        dload_file_location_name = get_download_files_by_name(
            names=parsed_name_filters, file_locations=file_locations
        )
        download_file_locations = dload_file_location_name
    if regexp:
        dload_file_location_regexp = get_download_files_by_regexp(
            regexp=regexp,
            file_locations=file_locations,
            filtered_files=download_file_locations if names else None,
        )
        download_file_locations = dload_file_location_regexp
    if ranges:
        parsed_range_filters = parse_parameters(ranges)
        dload_file_location_range = get_download_files_by_range(
            ranges=parsed_range_filters,
            file_locations=file_locations,
            filtered_files=download_file_locations if names or regexp else None,
        )
        download_file_locations = dload_file_location_range

    if names or regexp or ranges:
        if not download_file_locations:
            display_message(
                msg_type="error",
                msg="No files matching the filters",
            )
            sys.exit(1)
    else:
        download_file_locations = file_locations

    if dryrun:
        display_message(msg="\n".join(download_file_locations))
        sys.exit(0)

    total_files = len(download_file_locations)
    path = str(recid)
    if not os.path.isdir(path):
        try:
            os.mkdir(path)
        except OSError:
            display_message(
                msg_type="error",
                msg="Creation of the directory {} failed".format(path),
            )
    if not download_engine:
        if protocol.startswith("http"):
            download_engine = "requests"
        elif protocol == "xrootd":
            download_engine = "xrootd"
    for file_location in download_file_locations:
        display_message(
            msg_type="info",
            msg="Downloading file {} of {}".format(
                download_file_locations.index(file_location) + 1, total_files
            ),
        )
        download_single_file(
            path=path,
            file_location=file_location,
            protocol=protocol,
            download_engine=download_engine,
        )
        check_error(
            path=path,
            file_location=file_location,
            protocol=protocol,
            retry_limit=retry_limit,
            retry_sleep=retry_sleep,
        )
        if verify:
            file_info_remote = get_file_info_remote(
                server,
                recid,
                protocol=protocol,
                filtered_files=[file_location],
            )
            file_info_local = get_file_info_local(recid)
            verify_file_info(file_info_local, file_info_remote)
    display_message(
        msg_type="info",
        msg="Success!",
    )


@cernopendata_client.command()
@click.option("--recid", type=click.INT, help="Record ID")
@click.option(
    "--server",
    default=SERVER_HTTP_URI,
    type=click.STRING,
    help="Which CERN Open Data server to query? [default={}]".format(SERVER_HTTP_URI),
)
def verify_files(server, recid):
    """Verify downloaded data file integrity.

    Select a CERN Open Data bibliographic record by a record ID and
    verify integrity of downloaded data files belonging to this
    record.

    Examples: \n
    \t $ cernopendata-client verify-files --recid 5500
    """
    # Validate parameters
    validate_server(server)
    if recid is not None:
        validate_recid(recid)

    # Get remote file information
    file_info_remote = get_file_info_remote(server, recid)

    # Get local file information
    file_info_local = get_file_info_local(recid)
    if not file_info_local:
        display_message(
            msg_type="error",
            msg="No local files found for record {}. Perhaps run `download-files` first? Exiting.".format(
                recid
            ),
        )
        sys.exit(1)

    # Verify number of files
    display_message(
        msg_type="info",
        msg="Verifying number of files for record {}... ".format(recid),
    )
    display_message(
        msg_type="note",
        msg="Expected {}, found {}".format(len(file_info_remote), len(file_info_local)),
    )
    if len(file_info_remote) != len(file_info_local):
        display_message(
            msg_type="error",
            msg="File count does not match.",
        )
        sys.exit(1)

    # Verify size and checksum of each file
    verify_file_info(file_info_local, file_info_remote)

    # Success!
    display_message(
        msg_type="info",
        msg="Success!",
    )


@cernopendata_client.command()
@click.option(
    "-R",
    "--recursive",
    "recursive",
    is_flag=True,
    default=False,
    help="Iterate recursively in the given directory path",
)
@click.option(
    "--timeout",
    default=LIST_DIRECTORY_TIMEOUT,
    type=click.INT,
    help="Timeout after which to exit running the command [default={}]".format(
        LIST_DIRECTORY_TIMEOUT
    ),
)
@click.argument("path", type=click.STRING)
def list_directory(path, recursive, timeout):
    """List contents of a EOSPUBLIC Open Data directory.

    Returns the list of files and subdirectories of a given EOSPUBLIC directory.

    Examples: \n
    \t $ cernopendata-client list-directory /eos/opendata/cms/validated-runs/Commissioning10\n
    \t $ cernopendata-client list-directory /eos/opendata/cms/Run2010B/BTau/AOD --recursive\n
    \t $ cernopendata-client list-directory /eos/opendata/cms/Run2010B --recursive --timeout 10
    """
    validate_directory(directory=path)
    files = get_list_directory(path, recursive, timeout)
    if not files:
        display_message(
            msg_type="info",
            msg="No files in the directory.",
        )
        sys.exit(2)
    display_message(msg="\n".join(files))



# -*- coding: utf-8 -*-
# This file is part of cernopendata-client.
#
# Copyright (C) 2020 CERN.
#
# cernopendata-client is free software; you can redistribute it and/or modify
# it under the terms of the GPLv3 license; see LICENSE file for more details.

"""cernopendata-client file metadater related utilities."""

import json
import sys
from collections import Counter

from .printer import display_message


def handle_error_message(field):
    # noqa: D301
    """Handle error message.

    :param field: Name of the field to access

    :type field: str

    :return: None
    """
    display_message(
        msg_type="error",
        msg="Field '{}' is not present in metadata".format(field),
    )
    sys.exit(1)


def filter_matching_output(matching_objects, output_field, output_json):
    # noqa: D301
    """Filter matching objects based on specified criteria.

    :param possible: Dictionary containing matching objects
    :param output_field: Name of the array containing objects to access
    :param output_json: JSON containing metadata objects

    :type possible: dict
    :type field: str
    :type output_json: list or dict

    :return: None
    """
    index_frequency = Counter(
        [int(item.split("_")[1]) for item in matching_objects.keys()]
    )

    output_object_index, count = index_frequency.most_common(1)[0]
    output_object = output_json[output_object_index]
    if count == 1:
        for object in matching_objects.values():
            if output_field in object:
                display_message(msg=object[output_field])
            else:
                display_message(msg=json.dumps(object, indent=4))
    else:
        if output_field in output_object:
            display_message(msg=output_object[output_field])
        else:
            display_message(msg=json.dumps(output_object, indent=4))


def filter_metadata(output_field, filters, output_json):
    # noqa: D301
    """Filter metadata objects based on specified criteria.

    :param output_field: Name of the array containing objects to access
    :param filters: Argument of the --filter option in the format some_field_name=some_value
    :param output_json: JSON containing metadata objects

    :type field: str
    :type filters: str
    :type output_json: list or dict

    :return: None
    """
    matching_objects = {}
    for filter in filters:
        no_objects_found = True
        wrong_field = True

        filter_fields = filter.split("=")
        if len(filter_fields) != 2:
            display_message(
                msg_type="error",
                msg="Invalid filter format. Use --filter some_field_name=some_value",
            )
            sys.exit(1)
        filterField_name, filterField_value = filter_fields
        filterField_names = filterField_name.split(".")
        for index, object in enumerate(output_json):
            if object == "$schema":
                handle_error_message(output_field)
            if filterField_name in object:
                wrong_field = False
                if filterField_value == object[filterField_names[-1]]:
                    matching_objects[f"{filterField_names[-1]}_{index}"] = object
                    no_objects_found = False
        if no_objects_found and not wrong_field:
            display_message(
                msg_type="error",
                msg="No objects found with {}={}".format(
                    filterField_names[-1], filterField_value
                ),
            )
            sys.exit(1)
        elif wrong_field:
            handle_error_message(filterField_names[-1])
    if matching_objects:
        filter_matching_output(matching_objects, output_field, output_json)
